package com.sf.vertx.handle;

import java.util.Map;

import com.sf.vertx.api.pojo.Strategy;
import com.sf.vertx.constans.SacErrorCode;

import io.vertx.ext.web.RoutingContext;
import io.vertx.ext.web.handler.HttpException;
import lombok.extern.slf4j.Slf4j;

/***
 * redis存储
 * 
 * @author xy
 *
 */
@Slf4j
public class RateLimitHandlerRedisImpl implements RateLimitHandler {
	private RedisRateLimiter rateLimiter;
	// private int pattern;// 1:app,2:接口默认,3:服务接口配置限流策略

	public RateLimitHandlerRedisImpl(RedisRateLimiter rateLimiter) {
		this.rateLimiter = rateLimiter;
	}

	@Override
	public void handle(RoutingContext rc) {
		try {
			// TODO 测试异常
			String appCodeHeaderKey = rc.request().headers().get(AppConfigHandle.getAppCodeHeaderKey());
			String apiCodeHeaderKey = rc.request().headers().get(AppConfigHandle.getApiCodeHeaderKey());
			Strategy apiCodeStrategy = AppConfigHandle.getApiCurrentLimiting(appCodeHeaderKey + ":" + apiCodeHeaderKey);

			Strategy globalApiStrategy = AppConfigHandle.getGlobalApiCurrentLimitingConfig(appCodeHeaderKey);
			Strategy globalAppStrategy = AppConfigHandle.getGlobalAppCurrentLimitingConfig(appCodeHeaderKey);

			Strategy apiStrategy = apiCodeStrategy != null ? apiCodeStrategy
					: globalApiStrategy != null ? globalApiStrategy : null;
			Strategy appStrategy = globalAppStrategy != null ? globalAppStrategy : null;

			if (apiStrategy != null || appStrategy != null) {
				Map<Integer, Boolean> retMap = rateLimiter.acquire(rc, apiStrategy, appStrategy);

				if (apiStrategy != null && retMap.get(1) == false) {
					rc.fail(new HttpException(10015, apiStrategy.getDefaultResponse()));
					return;
				}

				if (appStrategy != null && retMap.get(2) == false) {
					rc.fail(new HttpException(10017, appStrategy.getDefaultResponse()));
					return;
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
			rc.fail(new HttpException(SacErrorCode.DEFAULT_ERROR_CODE));
			return;
		}
		rc.next();
		return;
	}
}
